@@ -3,6 +3,7 @@
 namespace PHPStan\PhpDocParser\Ast\Type;
 
 use PHPStan\PhpDocParser\Ast\NodeAttributes;
+use function array_map;
 use function implode;
 
 class IntersectionTypeNode implements TypeNode
@@ -11,8 +12,11 @@ class IntersectionTypeNode implements TypeNode
 	use NodeAttributes;
 
 	/** @var TypeNode[] */
-	public $types;
+	public array $types;
 
+	/**
+	 * @param TypeNode[] $types
+	 */
 	public function __construct(array $types)
 	{
 		$this->types = $types;
@@ -21,7 +25,13 @@ class IntersectionTypeNode implements TypeNode
 
 	public function __toString(): string
 	{
-		return '(' . implode(' & ', $this->types) . ')';
+		return '(' . implode(' & ', array_map(static function (TypeNode $type): string {
+			if ($type instanceof NullableTypeNode) {
+				return '(' . $type . ')';
+			}
+
+			return (string) $type;
+		}, $this->types)) . ')';
 	}
 
 }
